searchData={"items":[{"type":"module","title":"Sonyflakex","doc":"# Sonyflakex\n\nSonyflake is a distributed unique ID generator inspired by [Twitter's Snowflake](https://blog.twitter.com/2010/announcing-snowflake).  \n\nThis is an Elixir implementation of the original [sony/sonyflake](https://github.com/sony/sonyflake), which is written in Go.\n\nSonyflake focuses on lifetime and performance on many host/core environment.\nSo it has a different bit assignment from Snowflake.\nA Sonyflake ID is composed of\n\n    39 bits for time in units of 10 msec\n     8 bits for a sequence number\n    16 bits for a machine id\n\nAs a result, Sonyflake has the following advantages and disadvantages:\n\n- The lifetime (174 years) is longer than that of Snowflake (69 years)\n- It can work in more distributed machines (2^16) than Snowflake (2^10)\n- It can generate 2^8 IDs per 10 msec at most in a single machine/thread (slower than Snowflake)\n\nHowever, if you want more generation rate in a single host,\nyou can run multiple Sonyflake ID generators concurrently.","ref":"Sonyflakex.html"},{"type":"module","title":"Installation - Sonyflakex","doc":"Add `sonyflakex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:sonyflakex, \"~> 0.2.0\"}\n  ]\nend\n```\n\nThen update your dependencies with the following command:\n\n```\nmix deps.get\n```","ref":"Sonyflakex.html#module-installation"},{"type":"module","title":"Usage - Sonyflakex","doc":"Add `Sonyflakex` as one of your application's root supervisor child in `application.ex`.\n\n```elixir\ndefmodule MyApp do\n  use Application\n\n  @impl Application\n  def start(_type, _args) do\n    children = [\n      Sonyflakex,\n      # other dependencies \n    ]\n    Supervisor.start_link(children, strategy: :one_for_one)\n  end\nend\n```\n\nThis configuration will register the default `Sonyflakex` GenServer using the module name and you can generate a new ID by using the following call.\n\n```elixir\nSonyflakex.next_id()\n```","ref":"Sonyflakex.html#module-usage"},{"type":"module","title":"Limitations - Sonyflakex","doc":"Like the reference implementation in Go, the default `Sonyflakex` GenServer will pause the process execution for a few milliseconds in case the sequence number in the 10 ms windows overflows. This behaviour prevents the generation of duplicated IDs. However, if you need to generate more than 2^8 IDs in a 10 ms window of time, it can create a performance bottleneck for your system.\n\nIf you need to generate a higher volume of IDs in short periods of time, then you might need to run a pool of multiple `Sonyflakex` GenServers (each with a unique machine ID).","ref":"Sonyflakex.html#module-limitations"},{"type":"module","title":"Pending - Sonyflakex","doc":"- [ ] Callback to check machine ID is unique.","ref":"Sonyflakex.html#module-pending"},{"type":"module","title":"License - Sonyflakex","doc":"The MIT License (MIT)\n\nSee [LICENSE](https://github.com/elciok/sonyflakex/blob/main/LICENSE) for details.","ref":"Sonyflakex.html#module-license"},{"type":"function","title":"Sonyflakex.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Sonyflakex.html#child_spec/1"},{"type":"function","title":"Sonyflakex.next_id/0","doc":"Get new ID from running Sonyflake process.\n\nReturns: New integer ID.","ref":"Sonyflakex.html#next_id/0"},{"type":"function","title":"Sonyflakex.start_link/1","doc":"Starts GenServer process that generates\nSonyflake IDs.\n\nOptions:\n  - start_time: UNIX timestamp used as starting point\n      for other timestamps used to compose IDs\n  - machine_id: integer that identifies the machine\n      generating IDs. It is also part of the ID\n      so it should fit in 16 bits.\n\nReturns:\n  - `{:ok, pid}`: In case process is started successfully, it returns a tuple containing an ID.\n  - `{:error, error_detail}`: If the process can't be started due to invalid configuration options\n    it will return an error tuple containing details about the validation error.\n\nAn example of setting configuration options in an application:application:\n\n```elixir\ndefmodule MyApp do\n  use Application\n\n  @impl Application\n  def start(_type, _args) do\n    children = [\n      {Sonyflakex, machine_id: 33, start_time: 1712269128},\n      # other dependencies\n    ]\n    Supervisor.start_link(children, strategy: :one_for_one)\n  end\nend\n```","ref":"Sonyflakex.html#start_link/1"},{"type":"module","title":"Sonyflakex.Config","doc":"Configuration validation helpers","ref":"Sonyflakex.Config.html"},{"type":"function","title":"Sonyflakex.Config.set_default/3","doc":"Sets default value for an option if option is not set\nin option list. Default value is set from a callback.\n\nArgs:\n  - opts: Configuration options.\n  - option_name: Key for configuration option being validated.\n  - default_callback: Function that returns a default value.","ref":"Sonyflakex.Config.html#set_default/3"},{"type":"function","title":"Sonyflakex.Config.validate_bit_option_length/3","doc":"Validates option value integer can be set as a binary\nin a field with a limited number of bits.\n\nArgs:\n  - opts: Configuration options\n  - option_name: Key for configuration option being validated.\n  - max_bits: Maximum number of bits that would fit the option value.","ref":"Sonyflakex.Config.html#validate_bit_option_length/3"},{"type":"function","title":"Sonyflakex.Config.validate_is_integer/2","doc":"Validates option value is an integer. This is checked only\nif value is set.","ref":"Sonyflakex.Config.html#validate_is_integer/2"},{"type":"function","title":"Sonyflakex.Config.value_fits_in_bits/2","doc":"Checks if integer value binary representatino would fit in a number of bits.","ref":"Sonyflakex.Config.html#value_fits_in_bits/2"},{"type":"function","title":"Examples - Sonyflakex.Config.value_fits_in_bits/2","doc":"iex> Sonyflakex.Config.value_fits_in_bits(255, 8)\n    true","ref":"Sonyflakex.Config.html#value_fits_in_bits/2-examples"},{"type":"module","title":"Sonyflakex.Generator","doc":"Logic to generate new IDs.","ref":"Sonyflakex.Generator.html"},{"type":"function","title":"Sonyflakex.Generator.next_id/2","doc":"Returns new ID based on current state.\n\nArgs:\n  - `state`: Generator state.\n  - `utc_now`: (Optional) Function that returns current datetime\n  using the same contract as DateTime.utc_now/0. It is used to\n  mock datetime generation in tests.\n\nReturns:\n  - `{:ok, new_id, new_state}`: new ID value and state after generating the ID.\n\n  - `{:error, :overflow}`: if incrementing the sequence would overflow\n  the 8 bit field then it returns an error response.","ref":"Sonyflakex.Generator.html#next_id/2"},{"type":"module","title":"Sonyflakex.IpAddress","doc":"Helpers to handle IP address calculations.","ref":"Sonyflakex.IpAddress.html"},{"type":"function","title":"Sonyflakex.IpAddress.first_private_ipv4/1","doc":"Returns one of the machine's private IPv4 addresses.\n\nArgs:\n  - `list_ips_func`: Function that returns IP addresses. Should follow the\n  same contract as `:inet.getif/0`. Used in tests to mock getting addresses.","ref":"Sonyflakex.IpAddress.html#first_private_ipv4/1"},{"type":"function","title":"Examples - Sonyflakex.IpAddress.first_private_ipv4/1","doc":"iex> Sonyflakex.IpAddress.lower_16_bit_ip_address({192, 168, 1, 90})\n    346","ref":"Sonyflakex.IpAddress.html#first_private_ipv4/1-examples"},{"type":"function","title":"Sonyflakex.IpAddress.is_private_ipv4/1","doc":"Checks if tuple containing a IPv4 address\nis a private network address.\n\nArgs:\n  - `ip`: 4 item tuple containing the address octets.\n\nReturns: `true` if the address is used in private networks.","ref":"Sonyflakex.IpAddress.html#is_private_ipv4/1"},{"type":"function","title":"Examples - Sonyflakex.IpAddress.is_private_ipv4/1","doc":"iex> Sonyflakex.IpAddress.is_private_ipv4({192, 168, 0, 90})\n    true","ref":"Sonyflakex.IpAddress.html#is_private_ipv4/1-examples"},{"type":"function","title":"Sonyflakex.IpAddress.lower_16_bit_ip_address/1","doc":"Extracts two lower octets as a 16 bit integer.\n\nArgs:\n  - `ip`: 4 item tuple containing the address octets.\n\nReturns: Two lower octets as an integer.","ref":"Sonyflakex.IpAddress.html#lower_16_bit_ip_address/1"},{"type":"function","title":"Examples - Sonyflakex.IpAddress.lower_16_bit_ip_address/1","doc":"iex> Sonyflakex.IpAddress.lower_16_bit_ip_address({192, 168, 1, 90})\n    346","ref":"Sonyflakex.IpAddress.html#lower_16_bit_ip_address/1-examples"},{"type":"type","title":"Sonyflakex.IpAddress.t/0","doc":"Tuple containing octets that represent a IP v4 network address.","ref":"Sonyflakex.IpAddress.html#t:t/0"},{"type":"module","title":"Sonyflakex.State","doc":"Handles internal state data.","ref":"Sonyflakex.State.html"},{"type":"function","title":"Sonyflakex.State.create_state/4","doc":"Creates state tuple used internally by Sonyflakex.","ref":"Sonyflakex.State.html#create_state/4"},{"type":"function","title":"Examples - Sonyflakex.State.create_state/4","doc":"iex> Sonyflakex.State.create_state(1, 2, 3, 4)\n    {1, 2, 3, 4}","ref":"Sonyflakex.State.html#create_state/4-examples"},{"type":"function","title":"Sonyflakex.State.increment_sequence/1","doc":"Increments current sequence number and checks for overflow.\n\nReturns:\n  - `{:ok, new_sequence}`: if the sequence can be incremented\n  its new value is returned as the second element of the response.\n\n  - `{:error, :overflow}`: if incrementing the sequence would overflow\n  the 8 bit field then it returns an error response.","ref":"Sonyflakex.State.html#increment_sequence/1"},{"type":"function","title":"Examples - Sonyflakex.State.increment_sequence/1","doc":"iex> Sonyflakex.State.increment_sequence({1, 1, 1, 41})\n    {:ok, 42}\n\n    iex> Sonyflakex.State.increment_sequence({1, 1, 1, 255})\n    {:error, :overflow}","ref":"Sonyflakex.State.html#increment_sequence/1-examples"},{"type":"function","title":"Sonyflakex.State.new/1","doc":"Initializes ID generator state.\n\nThis method should be used by clients to create the initial state for\nthe ID generator with the following default configuration:\n\n  - *Start time*: '2014-09-01T00:00:00Z' is used when calculating\n  elapsed time for timestamps in state.\n  - *Machine ID*: Lower 16 bits of one of the machine's private\n  IP addresses. If you run multiple generators in the same\n  machine this field will be set to the same value and duplicated\n  IDs might be generated.","ref":"Sonyflakex.State.html#new/1"},{"type":"function","title":"Sonyflakex.State.to_id/1","doc":"Converts internal state to integer ID.","ref":"Sonyflakex.State.html#to_id/1"},{"type":"function","title":"Examples - Sonyflakex.State.to_id/1","doc":"iex> Sonyflakex.State.to_id({1, 2, 3, 4})\n    33816579","ref":"Sonyflakex.State.html#to_id/1-examples"},{"type":"type","title":"Sonyflakex.State.sonyflake_id/0","doc":"ID generated by Sonyflake","ref":"Sonyflakex.State.html#t:sonyflake_id/0"},{"type":"type","title":"Sonyflakex.State.t/0","doc":"Represents internal ID generator state used to compute the next ID.\n\nTuple composed by (in order):\n  1. *start_time*: Timestamp used as relative starting point relative to\n    elapsed time used in generated IDs.\n  2. *elapsed_time*: Timestamp in 10 ms unit of how much time has passed\n    since start_time.\n  3. *machine_id*: Value used to uniquely identify each machine generating IDs.\n  4. *sequence*: Number incremented for IDs generated in the same 10 ms time window.","ref":"Sonyflakex.State.html#t:t/0"},{"type":"module","title":"Sonyflakex.Time","doc":"Helpers to handle timestamp calculations.","ref":"Sonyflakex.Time.html"},{"type":"function","title":"Sonyflakex.Time.current_elapsed_time/2","doc":"Computes timestamp in unit of 10ms elapsed from start time.\n\nArgs:\n  - `start_time`: Timestamp in unit of 10 ms used as reference.\n  - `utc_now`: (Optional) Function that returns current DateTime\n  using the same contract as DateTime.utc_now/0. It is used to\n  mock datetime generation in tests.","ref":"Sonyflakex.Time.html#current_elapsed_time/2"},{"type":"function","title":"Sonyflakex.Time.default_epoch/0","doc":"Default timestamp used as start reference when computing elapsed time.","ref":"Sonyflakex.Time.html#default_epoch/0"},{"type":"function","title":"Examples - Sonyflakex.Time.default_epoch/0","doc":"iex> Sonyflakex.Time.default_epoch()\n    140952960000","ref":"Sonyflakex.Time.html#default_epoch/0-examples"},{"type":"function","title":"Sonyflakex.Time.time_until_next_timestamp/3","doc":"Computed how many milliseconds to wait from current time until\nthe next timestamp window from `elapsed_time`.\n\nIt is used to pause the process and wait until the next timestamp\nwindows to generate new IDs and avoid a sequence overflow.\n\nArgs:\n  - `start_time`: Timestamp in unit of 10ms used as base reference\n  for timestamps.\n  - `elapsed_time`: Timestamp in unit of 10ms to generate the\n  next ID.\n  - `current_time`: Current clock timestamp in ms.\n\nReturns: time in milliseconds until next elapsed time or zero\nif next elapsed time is in the past.","ref":"Sonyflakex.Time.html#time_until_next_timestamp/3"},{"type":"function","title":"Examples - Sonyflakex.Time.time_until_next_timestamp/3","doc":"iex> Sonyflakex.Time.time_until_next_timestamp(0, 1, 15)\n    5","ref":"Sonyflakex.Time.html#time_until_next_timestamp/3-examples"},{"type":"function","title":"Sonyflakex.Time.to_sonyflake_time/1","doc":"Converts DateTime to timestamp in unit of 10ms.","ref":"Sonyflakex.Time.html#to_sonyflake_time/1"},{"type":"function","title":"Examples - Sonyflakex.Time.to_sonyflake_time/1","doc":"iex> Sonyflakex.Time.to_sonyflake_time(~U[2020-10-01 00:00:01.020777Z])\n    160151040102","ref":"Sonyflakex.Time.html#to_sonyflake_time/1-examples"},{"type":"type","title":"Sonyflakex.Time.sonyflake_timestamp/0","doc":"Timestamp in 10ms unit used in Sonyflake IDs.","ref":"Sonyflakex.Time.html#t:sonyflake_timestamp/0"},{"type":"type","title":"Sonyflakex.Time.unix_timestamp_ms/0","doc":"UNIX timestamp in ms.","ref":"Sonyflakex.Time.html#t:unix_timestamp_ms/0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}